package main

import (
	"io"
	"log"
	"net"
	"os"
	"sync"
)

func main() {
	if len(os.Args) != 2 {
		log.Fatalf("usage: %s IP:PORT", os.Args[0])
	}
	addr := os.Args[1]
	if err := run(addr); err != nil {
		log.Fatal(err)
	}
}

func run(addr string) error {
	conn, err := net.Dial("tcp", addr)
	if err != nil {
		return err
	}
	defer conn.Close()
	log.Printf("%v: begin", conn.RemoteAddr())
	defer log.Printf("%v: end", conn.RemoteAddr())

	handleConnection(conn)
	return nil
}

// Read from the connection and write to stdout; read from stdin and write to
// the connection.
func handleConnection(conn net.Conn) {
	var wg sync.WaitGroup
	wg.Add(2)
	go func() {
		_, err := io.Copy(os.Stdout, conn)
		if err != nil {
			log.Printf("%v: error copying to stdout: %v", conn.RemoteAddr(), err)
		}
		os.Stdout.Close()
		wg.Done()
	}()
	go func() {
		_, err := io.Copy(conn, os.Stdin)
		if err != nil {
			log.Printf("%v: error copying from stdin: %v", conn.RemoteAddr(), err)
		}
		conn.(*net.TCPConn).CloseWrite()
		wg.Done()
	}()
	wg.Wait()
}
