package main

import (
	"io"
	"log"
	"net"
	"os"
)

func main() {
	if len(os.Args) != 2 {
		log.Fatalf("usage: %s IP:PORT", os.Args[0])
	}
	addr := os.Args[1]
	if err := run(addr); err != nil {
		log.Fatal(err)
	}
}

func run(addr string) error {
	ln, err := net.Listen("tcp", addr)
	if err != nil {
		return err
	}
	defer ln.Close()
	return acceptConnections(ln)
}

func acceptConnections(ln net.Listener) error {
	for {
		conn, err := ln.Accept()
		if err != nil {
			if err, ok := err.(net.Error); ok && err.Temporary() {
				continue
			}
			return err
		}
		go func() {
			defer conn.Close()
			log.Printf("%v: begin", conn.RemoteAddr())
			defer log.Printf("%v: end", conn.RemoteAddr())
			err := handleConnection(conn)
			if err != nil {
				log.Printf("%v: handleConnection: %v", conn.RemoteAddr(), err)
			}
		}()
	}
}

func handleConnection(conn net.Conn) error {
	n, err := io.Copy(conn, conn)
	log.Printf("%v: echoed %v bytes", conn.RemoteAddr(), n)
	return err
}
