package turbotunnel

import (
	"crypto/rand"
	"encoding/hex"
)

// SessionID represents an ongoing session, independent of any network address.
// SessionID satisfies the net.Addr interface.
type SessionID [8]byte

// NewSessionID creates a new random SessionID.
func NewSessionID() SessionID {
	var id SessionID
	_, err := rand.Read(id[:])
	if err != nil {
		panic(err)
	}
	return id
}

func (id SessionID) Network() string { return "session" }
func (id SessionID) String() string  { return hex.EncodeToString(id[:]) }
